from PySide6.QtWidgets import (
    QDialog, QVBoxLayout, QHBoxLayout, QLabel, QLineEdit, QTextEdit, QPushButton, QComboBox, QFormLayout
)
from PySide6.QtGui import QRegularExpressionValidator
from PySide6.QtCore import QRegularExpression
from models.book import Book, BookStatusEnum

class BookEditor(QDialog):
    def __init__(self, book: Book, parent=None):
        super().__init__(parent)

        self.book = book
        self.setWindowTitle("Edit Book")
        self.setMinimumSize(400, 300)

        # Create main layout
        layout = QVBoxLayout(self)

        # Form layout for book fields
        form_layout = QFormLayout()

        # Title field
        self.title_input = QLineEdit(self.book.title)
        form_layout.addRow("Title:", self.title_input)

        full_author_name = f"{self.book.author.first_name} {self.book.author.last_name}"
        self.author_input = QLineEdit(full_author_name)
        form_layout.addRow("Author: ", self.author_input)

        # Description field
        self.description_input = QTextEdit(self.book.description)
        form_layout.addRow("Description:", self.description_input)

        # Year published field
        self.year_input = QLineEdit(self.book.year_published)
        # self.year_input.setValidator
        form_layout.addRow("Year Published:", self.year_input)

        # ISBN field
        self.isbn_input = QLineEdit(self.book.isbn)
        self.isbn_expression = QRegularExpression("\d{10}|\d{13}")
        self.isbn_validator = QRegularExpressionValidator(self.isbn_expression)
        self.isbn_input.setValidator(self.isbn_validator)
        form_layout.addRow("ISBN:", self.isbn_input)

        all_categories = ", ".join(category.name for category in self.book.categories)
        self.categories_input = QLineEdit(all_categories)
        form_layout.addRow("Categories: ", self.categories_input)

        # Status dropdown
        self.status_input = QComboBox()
        self.status_input.addItems([status.value for status in BookStatusEnum])
        self.status_input.setCurrentText(self.book.status.value)
        form_layout.addRow("Status:", self.status_input)

        layout.addLayout(form_layout)

        # Buttons
        button_layout = QHBoxLayout()

        self.save_button = QPushButton("Save")
        self.save_button.clicked.connect(self.save_book)
        button_layout.addWidget(self.save_button)

        self.cancel_button = QPushButton("Discard")
        self.cancel_button.clicked.connect(self.reject)
        button_layout.addWidget(self.cancel_button)

        layout.addLayout(button_layout)

    def save_book(self):
        # Update book object with input values
        self.book.title = self.title_input.text()
        self.book.description = self.description_input.toPlainText()
        self.book.year_published = self.year_input.text()
        self.book.isbn = self.isbn_input.text()
        self.book.status = BookStatusEnum(self.status_input.currentText())

        # Accept the dialog and close
        self.accept()
